/**
 * Simulation of process scheduling algorithms
 *
 * Ron Mak
 * Department of Computer Engineering
 * San Jose State University
 */
#ifndef HPF_H_
#define HPF_H_

#include "SchedulingAlgorithm.h"

const int PRIORITY_QUEUE_COUNT = 4;

/**
 * Abstract base class for High Priority First (HPS) algorithm.
 */
class HPF: public SchedulingAlgorithm
{
public:
    HPF(int rcount, int pcount, int qlimit)
        : SchedulingAlgorithm(rcount, pcount, qlimit) {}

    virtual ~HPF() {}

protected:
    vector<Process *> priority_queues[PRIORITY_QUEUE_COUNT];

    /**
     * Age a process in a priority queue by bumping it up
     * to the next higher queue after it has been in the
     * same queue for MAX_AGE time quanta.
     * @param quantum the time quantum when this occurs.
     */
    void age_processes(int quantum);
};

#endif /* HPF_H_ */
