/**
 * Simulation of process scheduling algorithms
 *
 * Ron Mak
 * Department of Computer Engineering
 * San Jose State University
 */

#include <iostream>
#include "HPF_NP.h"

using namespace std;

void HPF_NP::prepare_run(int run_index)
{
    cout << endl;
    cout << "------------------------------------------------" << endl;
    cout << "Highest Priority First - Non-preemptive (HPF_NP): run #"
         << run_index << endl;
    cout << "------------------------------------------------" << endl;
}

void HPF_NP::run_algorithm()
{
    int quantum = 0;

    for (int i = 0; i < PRIORITY_QUEUE_COUNT; i++) priority_queues[i].clear();

    // Run all the processes up to the quantum limit.
    do
    {
        // Append any newly arrived jobs to the
        // back of the appropriate priority queue.
        while (   (job_queue.size() > 0)
               && (quantum >= job_queue.front()->arrival_time))
        {
            Process *p = job_queue.front();
            p->stats.wait_time += quantum - p->arrival_time;
            p->start_time = quantum;
            p->previous_quantum = quantum - 1;

            priority_queues[p->priority - 1].push_back(p);
            job_queue.erase(job_queue.begin());
        }

        // Look for the highest priority process.
        Process *p = nullptr;
        for (int i = 0; (i < 4) && (p == nullptr); i++)
        {
            vector<Process *> *pq = &priority_queues[i];
            if (pq->size() > 0)
            {
                p = pq->front();
                pq->erase(pq->begin());
            }
        }

        if (p == nullptr)
        {
            timeline += '-';
            quantum++;
        }
        else
        {
            // Run the process.
            double exp_rt = p->expected_run_time;
            p->stats.wait_time += quantum - p->arrival_time;
            end_time = quantum + exp_rt;

            // Take over the CPU until done.
            do
            {
                timeline += p->name;
                exp_rt -= 1.0;

                age_processes(quantum);
                quantum++;
            } while (exp_rt > 0.0);

            p->stats.turnaround_time = p->stats.wait_time + p->expected_run_time;
            completed.push_back(p);
        }
    } while (quantum <= quantum_limit);
}
