/**
 * Simulation of process scheduling algorithms
 *
 * Ron Mak
 * Department of Computer Engineering
 * San Jose State University
 */
#include <iostream>
#include "HPF_P.h"

using namespace std;

void HPF_P::prepare_run(int run_index)
{
    cout << endl;
    cout << "-------------------------------------------" << endl;
    cout << "Highest Priority First - Preemptive (HPF_P): run #"
         << run_index << endl;
    cout << "-------------------------------------------" << endl;
}

void HPF_P::run_algorithm()
{
    Process *p = nullptr;
    int quantum = 0;

    for (int i = 0; i < PRIORITY_QUEUE_COUNT; i++) priority_queues[i].clear();

    // Run all the processes up to the quantum limit.
    do
    {
        // Append any newly arrived jobs to the
        // back of the appropriate priority queue.
        while (   (job_queue.size() > 0)
               && (quantum >= job_queue.front()->arrival_time))
        {
            p = job_queue.front();
            p->stats.wait_time += quantum - p->arrival_time;
            p->start_time = quantum;
            p->previous_quantum = quantum - 1;

            priority_queues[p->priority - 1].push_back(p);
            job_queue.erase(job_queue.begin());
        }

        vector<Process *> *hpq = nullptr;
        p = nullptr;

        // If it's past the quantum limit, only look
        // at processes that have already started
        // and remove the others.
        for (int i = 0; (i < PRIORITY_QUEUE_COUNT) && (p == nullptr); i++)
        {
            hpq = &priority_queues[i];

            if (hpq->size() > 0)
            {
                p = hpq->front();

                if (quantum > quantum_limit)
                {
                    while (   (p != nullptr)
                           && (p->expected_run_time == p->remaining_time))
                    {
                        hpq->erase(hpq->begin());
                        p = (hpq->size() > 0) ? hpq->front() : nullptr;
                    }
                }
            }
        }

        if (p == nullptr)
        {
            timeline += '-';
            quantum++;
        }
        else
        {
            // Run the process.
            p->stats.wait_time += quantum - p->previous_quantum - 1;
            p->previous_quantum = quantum;
            timeline += p->name;

            // Process finished?
            if (p->remaining_time <= 1.0)
            {
                // Yes: Remove from the priority queue.
                hpq->erase(hpq->begin());
                completed.push_back(p);
                p->stats.turnaround_time = quantum + p->remaining_time
                                                            - p->arrival_time;
                end_time = quantum + p->remaining_time;
            }
            else
            {
                // No: Move to the back of the priority queue.
                p->remaining_time -= 1;
                hpq->push_back(p);
                hpq->erase(hpq->begin());
            }

            age_processes(quantum);
            quantum++;
        }
    } while ((p != nullptr) || (quantum <= quantum_limit));

    // Erase extraneous trailing -
    if (timeline.back() == '-') timeline.erase(timeline.length() - 1);
}
