/**
 * Simulation of process scheduling algorithms
 *
 * Ron Mak
 * Department of Computer Engineering
 * San Jose State University
 */
#ifndef PROCESS_H_
#define PROCESS_H_

#include <vector>
#include "ProcessStats.h"

using namespace std;

// Highest priority first (HPF):
// How long to stay at a priority level
// before being bumped to the next higher level.
const int MAX_AGE = 5;

/**
 * A process object for the simulation.
 */
class Process
{
public:
    Process(int priority, double arrival_time, double expected_run_time)
        : name(' '), priority(priority), age(MAX_AGE),
          arrival_time(arrival_time), expected_run_time(expected_run_time),
          remaining_time(expected_run_time), start_time(0), previous_quantum(0)
    {}

    virtual ~Process() {}

public:
    char name;
    int priority;
    int age;
    double arrival_time;
    double expected_run_time;
    double remaining_time;
    double start_time;
    double previous_quantum;

    ProcessStats stats;
};

#endif /* PROCESS_H_ */
