#ifndef SCHEDULINGALGORITHM_H_
#define SCHEDULINGALGORITHM_H_

#include <string>
#include "Process.h"
#include "RunStats.h"

using namespace std;

/**
 * Abstract base class for process scheduling algorithms.
 */
class SchedulingAlgorithm
{
public:
    SchedulingAlgorithm(int rcount, int pcount, int qlimit)
        : run_count(rcount), process_count(pcount), quantum_limit(qlimit),
          start_time(0), end_time(0), timeline("") {}
    virtual ~SchedulingAlgorithm() {}

    /**
     * Run a simulation run_count times.
     */
    void run_simulations();

protected:
    int run_count;
    int process_count;
    int quantum_limit;

    double start_time;
    double end_time;

    string timeline;
    RunStats stats;

    vector<Process *> job_queue;    // all jobs for the simulation run
    vector<Process *> ready_queue;  // processes ready to go
    vector<Process *> completed;    // process completed during the run

    virtual void prepare_run(int run_index) = 0;
    virtual void run_algorithm() = 0;

    void initialize_job_queue();
    void print_job_queue();
    void print_timeline();
    void print_statistics(int run_index);
};

#endif /* SCHEDULINGALGORITHM_H_ */
